package common.caching;

import java.text.DecimalFormat;
import java.util.Hashtable;
import java.util.Calendar;
import java.util.Iterator;
import java.util.*;
import java.io.IOException;

import org.apache.jcs.JCS;
import org.apache.jcs.access.CacheAccess;
import org.apache.jcs.engine.ElementAttributes;
import org.apache.jcs.engine.control.CompositeCache;
import org.apache.jcs.engine.control.CompositeCacheManager;

import common.caching.CachingException;
import common.caching.util.CacheTimer;
import common.logging.WebPortalLogger;

/**
 * This is the base implementation of CachingManager interface. It implements all the
 * methods defined in CachingManager.
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 10/03/2003
 */
public abstract class BaseCacheManager implements ICacheManager {

   protected CacheList cacheList = CacheList.getInstance();

	protected static BaseCacheManager instance;
   protected String regionName;
   protected String DEFAULT_REGION = CacheRegions.DEFAULT_REGION;

	protected double readFromCache = 0;
	protected double readFromDataSource = 0;

	/**
 	 * Gets hit rate as percentage.
	 */
	public String getHitRate() {
      double value = (readFromCache/(readFromCache+readFromDataSource));
      String pattern = "###.##";
      DecimalFormat formatter = new DecimalFormat(pattern);
      String hitRate = formatter.format(value);

      return hitRate;
	}

	/**
 	 * Retrieves a cached object. Default to look in the cache.
	 */
	public Object getCacheObject(Object cacheKey, ICacheLoader cacheLoader) throws CachingException {
		return getCacheObject(cacheKey, true, cacheLoader);
	}

   /**
    * Retrieves a cached object. Second argument decides whether to look
	 * in the cache. Default to look in default cache region.
    */
   public Object getCacheObject(Object cacheKey, boolean fromCache, ICacheLoader cacheLoader) throws CachingException {
      return getCacheObject(cacheKey, fromCache, cacheLoader, DEFAULT_REGION);
   }

   /**
    * Retrieves a cached object. Default to look in the cache
    * and in default cache region.
    */
   public Object getCacheObject(Object cacheKey, ICacheLoader cacheLoader, String regionName) throws CachingException {
      return getCacheObject(cacheKey, true, cacheLoader, regionName);
   }

   /**
    * Stores CacheObjects in database using default cache region.
    * Clears old items and caches new.
    */
   public void putCacheObject(Object cacheKey, Object cacheObject) throws CachingException {
      putCacheObject(cacheKey, cacheObject, DEFAULT_REGION);
   }

   /**
    * Stores CacheObjects in the default cache region with the specified
    * day/time to expire the cahed object.
    */
   public void putCacheObject(Object cacheKey, Object cacheObject, boolean expireSchedule) throws CachingException {
      putCacheObject(cacheKey, cacheObject, expireSchedule, DEFAULT_REGION);
   }

   /**
    * Peeks into the cache using default cache region
    * for a given key without loading the value on demand.
    * If the value is cached, then it will be returned.
    * If the value is not already cached, then null is returned.
    */
   public Object peek(Object cacheKey) throws CachingException {
      return peek(cacheKey, DEFAULT_REGION);
   }

   /**
    * Check the cache using default cache region
    * for a given key.
    * If the value is cached, return true.
    * If the value is not already cached, return false.
    */
   public boolean isPresent(Object cacheKey) throws CachingException {
      String regionName = DEFAULT_REGION;
      return isPresent(cacheKey, regionName);
   }

	/**
    * Check the cache using the specified cache region
    * for a given key.
    * If the value is cached, return true.
    * If the value is not already cached, return false.
	 */
	public boolean isPresent(Object cacheKey, String regionName) throws CachingException {
      if (peek(cacheKey, regionName) != null) return true;
      else return false;
   }

	/**
	 * Remove the cached object from default cache region.
	 */
   public void removeCacheObject(Object cacheKey) throws CachingException {
      removeCacheObject(cacheKey, DEFAULT_REGION);
   }

	/**
	 * Remove all the cached objects from default cache region.
	 */
   public void clear() throws CachingException {
      clear(DEFAULT_REGION);
   }

}
// End of BaseCacheManager.java
