package common.caching;

import java.util.Properties;

import com.opensymphony.oscache.general.GeneralCacheAdministrator;
import com.opensymphony.oscache.base.Cache;
import com.opensymphony.oscache.base.CacheEntry;
import com.opensymphony.oscache.base.Config;
import com.opensymphony.oscache.base.NeedsRefreshException;
import com.opensymphony.oscache.base.algorithm.AbstractConcurrentReadCache;

import common.logging.WebPortalLogger;

/**
 * This is the base implementation of CachingManager interface. It implements all the
 * methods defined in CachingManager.
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 10/03/2003
 */
public class BaseCacheManagerOSCache extends BaseCacheManager implements ICacheManager {

   private GeneralCacheAdministrator cacheAdmin = new GeneralCacheAdministrator();
   private AbstractConcurrentReadCache cacheMap = null;
   private static final Properties cacheProperties = new Config().getProperties();

	// CONSTRUCTORS
	private BaseCacheManagerOSCache() {
		try {
         regionName = DEFAULT_REGION;
         String algorithmClass = cacheProperties.getProperty("cache.algorithm");
         WebPortalLogger.info("algorithmClass : " + algorithmClass);
         cacheMap = (AbstractConcurrentReadCache) Class.forName(algorithmClass).newInstance();
         WebPortalLogger.info("cacheMap : " + cacheMap);
		} catch (Exception e) {
			// Handle cache region initialization failure
         WebPortalLogger.info("Error in creating JCS instance.");
         WebPortalLogger.printStackTrace(e);
		}
	}

	/**
 	 * Singleton access point to the manager.
	 */
	public static ICacheManager getInstance() {
		if (instance == null) {
//			synchronized (BaseCacheManagerOSCache.class) {
//				if (instance == null) {
					instance = new BaseCacheManagerOSCache();
//				}
//			}
		}

//		synchronized (instance) {
//			instance.checkedOut++;
//		}

		return instance;
	}

   private Cache getCacheForRegion(String regionName) throws CachingException {

      try {
         Cache cache = cacheAdmin.getCache();
         return cache;
      } catch (Exception e) {
         WebPortalLogger.info("Exception in getCacheForRegion(): " + e.toString());
         throw new CachingException(e);
      }
   }

	/**
	 * Retrieves a CacheObject. Returns a new value object if one can't be
	 * loaded from the database. Database cache synchronization is
	 * handled by removing cache elements upon modification.
	 */
   public Object getCacheObject(Object cacheKey, boolean fromCache, ICacheLoader cacheLoader, String regionName) throws CachingException {
      Object cacheObject = null;
      Object cacheObjectKey = null;
      Cache cache = getCacheForRegion(regionName);

      long t1 = System.currentTimeMillis();

      // Check if cacheKey is an instance of CacheKey interface
      if (cacheKey instanceof ICacheKey) {
         cacheObjectKey = ((ICacheKey)cacheKey).getCacheKey();
      } else {
         cacheObjectKey = cacheKey;
      }

      cacheProperties.list(System.out);

      int refreshPeriod = Integer.parseInt(cacheProperties.getProperty("cache.refresh.period"));

      try {
         // First, if requested, attempt to load from cache
         if (fromCache) {
            cacheObject = cache.getFromCache(String.valueOf(cacheObjectKey), refreshPeriod);
            readFromCache++;
         }
      } catch (NeedsRefreshException nre) {
//         WebPortalLogger.printStackTrace(nre);
      }

      try {
         // Either fromCache was false or the object was not found, so
         // call loadCacheObject to create it
         if (cacheObject == null) {
            String cacheLoaderClassName = cacheLoader.getClass().getName();
            WebPortalLogger.info("Could not find the object in cache for key " + cacheObjectKey);
            WebPortalLogger.info("Calling loadCacheObject() in CacheLoader : " + cacheLoaderClassName);

            Object cacheLoaderObject = cacheList.getCacheLoader(cacheLoaderClassName);

            if (cacheLoaderObject == null) {
               String msg = "cacheLoaderObject is null for " + cacheLoaderClassName;
               WebPortalLogger.info(msg);
               WebPortalLogger.info("cacheLoader : " + cacheLoader);
               cacheList.putCacheLoader(cacheLoader);
               cacheLoaderObject = cacheList.getCacheLoader(cacheLoaderClassName);
            }
            cacheObject = ((ICacheLoader)cacheLoaderObject).loadCacheObject(cacheObjectKey, regionName);

            // store the object in the cache.
            cache.putInCache(String.valueOf(cacheObjectKey), cacheObject, new String[]{regionName});
            readFromDataSource++;
         } else {
            WebPortalLogger.info("Found the object in cache for cacheKey " + cacheKey.toString());
         }

         long t2 = System.currentTimeMillis();
         WebPortalLogger.info("Time taken to get cached object is : " + (t2-t1) + " milli-seconds.");

      } catch (CachingException e) {
         throw e;
      } catch (Exception e) {
         throw new CachingException("Exception in getCacheObject()", e);
      }
      return cacheObject;
   }

	/**
    * Stores CacheObjects in database. Clears old items and caches
    * new.
    */
	public void putCacheObject(Object cacheKey, Object cacheObject, String regionName) throws CachingException {
      Object cacheObjectKey = null;
		try {
         Cache cache = getCacheForRegion(regionName);

			// since any cached data is no longer valid, we should
			// remove the item from the cache if it's an update.
         cache.flushEntry(String.valueOf(cacheKey));

         // Check if cacheKey is an instance of CacheKey interface
         if (cacheKey instanceof ICacheKey) {
            cacheObjectKey = ((ICacheKey)cacheKey).getCacheKey();
         } else {
            cacheObjectKey = cacheKey;
         }
			// put the new object in the cache
         cache.putInCache(String.valueOf(cacheObjectKey), cacheObject, new String[]{regionName});
		} catch (Exception e) {
			// Handle failure in removing object or putting object to cache.
         e.printStackTrace();
         WebPortalLogger.info("Error in putting object into cache");
         throw new CachingException("Error in putting object into cache");
		}
	}

   /**
    * Stores CacheObjects in the cache with the specified
    * day/time to expire the cahed object.
    */
   public void putCacheObject(Object cacheKey, Object cacheObject, boolean expireSchedule, String regionName) throws CachingException {
      Object cacheObjectKey = null;
      try {
         Cache cache = getCacheForRegion(regionName);

         // since any cached data is no longer valid, we should
         // remove the item from the cache if it's an update.
         cache.flushEntry(String.valueOf(cacheObject));

         // Check if cacheKey is an instance of CacheKey interface
         if (cacheKey instanceof ICacheKey) {
            cacheObjectKey = ((ICacheKey)cacheKey).getCacheKey();
         } else {
            cacheObjectKey = cacheKey;
         }
         // put the new object in the cache
         cache.putInCache(String.valueOf(cacheObjectKey), cacheObject);
         cacheList.putScheduleExpiry(regionName);
      } catch (Exception e) {
         // Handle failure in removing object or putting object to cache.
         WebPortalLogger.info("Error in removing object or putting object to cache");
         throw new CachingException("Error in removing object or putting object to cache");
      }
   }

	/**
	 * Peeks into the cache for a given key without loading
    * the value on demand. If the value is cached, then it
    * will be returned. If the value is not already cached,
    * then null is returned.
	 */
	public Object peek(Object cacheKey, String regionName) throws CachingException {
      Object cacheObject = null;
      Cache cache = getCacheForRegion(regionName);

      Object cacheObjectKey = null;
      // Check if cacheKey is an instance of CacheKey interface
      if (cacheKey instanceof ICacheKey) {
         cacheObjectKey = ((ICacheKey)cacheKey).getCacheKey();
      } else {
         cacheObjectKey = cacheKey;
      }

      try {
         if (cache.getFromCache((String)cacheObjectKey, CacheEntry.INDEFINITE_EXPIRY) == null) {
            WebPortalLogger.info("Object does not exist in " + regionName + " region.");
         } else {
            WebPortalLogger.info("Object exists in the cache in " + regionName + " region.");
            cacheObject = cache.getFromCache(String.valueOf(cacheObjectKey), CacheEntry.INDEFINITE_EXPIRY);
         }
      } catch (NeedsRefreshException nre) {
         cache.putInCache((String)cacheObjectKey, null, new String[]{regionName});
      }

      return cacheObject;
	}

	/**
	 * Remove the cached object from the specified cache region.
	 */
   public void removeCacheObject(Object cacheKey, String regionName) throws CachingException {
      try {
         Cache cache = getCacheForRegion(regionName);
         cache.flushEntry((String)cacheKey);
      } catch (Exception e) {
         WebPortalLogger.info("Error in removing CacheObject from region : " + regionName);
         throw new CachingException("Error in removing CacheObject : " + regionName);
      }
   }

	/**
	 * Remove all the cached objects from the specified
	 * cache region.
	 */
	public void clear(String regionName) throws CachingException {
      WebPortalLogger.info("Removing all elements from cache region : " + regionName);
		try {
         Cache cache = getCacheForRegion(regionName);
         cache.flushGroup(regionName);
         readFromCache = 0;
         readFromDataSource = 0;
		} catch (Exception e) {
         WebPortalLogger.info("Error in clearing the cache.");
			throw new CachingException("Error in clearing the cache.");
		}
	}

	/**
	 * Set the default region cache to null.
	 */
   public void clearCache() throws CachingException {
      Cache cache = getCacheForRegion(regionName);
      cache = null;
   }

	/**
	 * Set the specified region cache to null.
	 */
   public void clearCache(String regionName) throws CachingException {
      Cache cache = getCacheForRegion(regionName);
		cache = null;
	}

}
// End of BaseCacheManagerOSCache.java

