package common.caching.test;

import java.util.*;
import java.io.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.extensions.ActiveTestSuite;

import common.logging.WebPortalLogger;
import common.caching.CachingException;
import common.caching.*;


/**
 * JUnit test case to test the caching methods defined in CachingManager interface.
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 10/03/2003
 */
public abstract class CachingTestCase extends TestCase implements ICacheLoader {

  /**
   * Attributes
   */
   protected static long timeToSleep;
   protected static String defaultCacheRegion     = CacheRegions.DEFAULT_REGION;
   protected static String testCacheRegion        = CacheRegions.TEST_CACHE_REGION;
   protected static String memoryCacheRegion      = CacheRegions.MEMORY_CACHE_REGION;

   protected static Random random = new Random(System.currentTimeMillis());
   protected static ICacheManager manager;

   /**
    * Constructor for the CachingTestCase object.
    */
   public CachingTestCase(String name) {
      super(name);
   }

   /**
    * Test tearDown
    */
   protected void tearDown() {
      WebPortalLogger.info("Inside tearDown() method. Set cache manager to null.");
      manager = null;
   }

   /**
    * runTests method
    */
   public void runTests() throws CachingException {
      System.out.println("BEGIN runTests()");

      testBasicCaching();
      testCachePerformance();
      testLoadFromCache();
      testStoreRetrieveClear();
      testMultipleCacheRegions();
      testCacheExpiry();
      testCachingUsingCustomCacheKey();
      testLRUMemoryCache();
      testMultipleStoreRetrieveClear();

      System.out.println("END runTests()");
   }

   /**
    * Data access logic implementation.
    * @param cacheKey Cache object
    * @param regionName Name of cache region
    * @return cached object
    * @throws CachingException
	 */
   public Object loadCacheObject(Object cacheKey, String regionName) throws CachingException {
      List list = null;

      if (cacheKey instanceof TestCacheKey) {
         // TestCacheKey
         TestCacheKey testCacheKey = new TestCacheKey();
         return testCacheKey.getData();
      } else if (cacheKey instanceof TestVehicleCacheKey) {
         // VechicleCacheTestKey
         TestVehicleCacheKey vehCacheTestKey = new TestVehicleCacheKey();
         return vehCacheTestKey.getData();
      } else {
         try {
            list = buildList();
         } catch (Exception e) {
            WebPortalLogger.info("Exception : " + e.toString());
            throw new CachingException(e);
         }
         return list;
      }
   }

   private LinkedList buildList() {
      LinkedList list = new LinkedList();
      for (int i = 0; i < 2000; i++) {
         list.add(buildMap());
      }
      return list;
   }

   private HashMap buildMap() {
      HashMap map = new HashMap();

//      byte[] keyBytes = new byte[32];
//      byte[] valBytes = new byte[128];
      int keyBytes = 0;
      int valBytes = 0;

      for (int i = 0; i < 100; i++) {
//         random.nextBytes(keyBytes);
//         random.nextBytes(valBytes);
         keyBytes = random.nextInt();
         valBytes = random.nextInt();

         map.put((new Integer(keyBytes)).toString(), (new Integer(valBytes)).toString());
      }
      return map;
   }

   //
   // TEST METHODS
   //
   public void testBasicCaching() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testBasicCaching() started.        ****");
      String key = "TEST123";

      try {
         LinkedList list = buildList();
         manager.putCacheObject(key, list);
         assertEquals(list, manager.getCacheObject(key, this));

         manager.clear();
      } catch (Exception e) {
         e.printStackTrace();
         throw new CachingException(e);
      }
      WebPortalLogger.info("****   TEST METHOD: testBasicCaching() completed.      ****");
   }

   // Test Method
   public void testCachePerformance() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testCachePerformance() started.        ****");

      try {
         long time = System.currentTimeMillis();
         String key = "LargeObject";

         Object costly = manager.getCacheObject(key, this);
         WebPortalLogger.info("First invocation took : " +
                              (System.currentTimeMillis() - time) + " ms.");

         time = System.currentTimeMillis();
         for (int i = 0; i < 10; i++) {
            costly = manager.getCacheObject(key, this);
         }
         WebPortalLogger.info("10 next invocations took : " +
                              (System.currentTimeMillis() - time) + " ms.");

         manager.clear();
      } catch (CachingException e) {
         e.printStackTrace();
         WebPortalLogger.info(e.toString());
         throw e;
      }
      catch (Exception e) {
         e.printStackTrace();
         throw new CachingException(e);
      }

      WebPortalLogger.info(
         "****   TEST METHOD: testCachePerformance() completed.      ****");
   }

   // Test Method
   public void testLoadFromCache() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testLoadFromCache() started.        ****");

      List list = null;
      String key = "TEST123";
      list = (LinkedList)manager.getCacheObject(key, this);

      try {
         timeToSleep = 3000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      assertTrue(true);
      manager.clear();
      WebPortalLogger.info("****   TEST METHOD: testLoadFromCache() completed.      ****");
   }

   // Test Method
   public void testStoreRetrieveClear() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testStoreRetrieveClear() started.        ****");

      Serializable key1 = "key 1";
      Serializable value1 = "value 1";
      Serializable key2 = "key 2";
      Serializable value2 = "value 2";
      Serializable key3 = "key 3";
      Serializable value3 = "value 3";

      assertTrue(!manager.isPresent(key1));
      manager.putCacheObject(key1, value1);
      assertTrue(manager.isPresent(key1));
      assertEquals(value1, manager.peek(key1));

      assertTrue(!manager.isPresent(key2));
      manager.putCacheObject(key2, value2);
      assertTrue(manager.isPresent(key2));
      assertEquals(value2, manager.peek(key2));

      assertTrue(!manager.isPresent(key3));
      manager.putCacheObject(key3, value3);
      assertTrue(manager.isPresent(key3));
      assertEquals(value3, manager.peek(key3));

      manager.removeCacheObject(key1);
      assertTrue(!manager.isPresent(key1));
      assertTrue(manager.isPresent(key2));
      assertTrue(manager.isPresent(key3));

      manager.removeCacheObject(key2);
      assertTrue(!manager.isPresent(key1));
      assertTrue(!manager.isPresent(key2));
      assertTrue(manager.isPresent(key3));

      manager.removeCacheObject(key3);
      assertTrue(!manager.isPresent(key1));
      assertTrue(!manager.isPresent(key2));
      assertTrue(!manager.isPresent(key3));

      // now clear all the objects in the cache
      manager.clear();

      WebPortalLogger.info("****   TEST METHOD: testStoreRetrieveClear() completed.      ****");
   }

   // Test Method
   public void testMultipleCacheRegions() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testMultipleCacheRegions() started.        ****");

      List list = null;
      String key = "TEST123";

      list = (LinkedList)manager.getCacheObject(key, this, testCacheRegion);
      assertNotNull(manager.peek(key, testCacheRegion));

      list = (LinkedList)manager.getCacheObject(key, this, memoryCacheRegion);
      assertNotNull(manager.peek(key, memoryCacheRegion));

      try {
         timeToSleep = 5000;
			WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      assertNotNull(manager.peek(key, testCacheRegion));
      assertNotNull(manager.peek(key, memoryCacheRegion));

      try {
         timeToSleep = 4000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      assertNotNull(manager.peek(key, testCacheRegion));
      assertNotNull(manager.peek(key, memoryCacheRegion));

      try {
         timeToSleep = 3000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      assertNotNull(manager.peek(key, testCacheRegion));
      assertNotNull(manager.peek(key, memoryCacheRegion));

      try {
         timeToSleep = 2000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      assertNotNull(manager.peek(key, testCacheRegion));
      assertNotNull(manager.peek(key, memoryCacheRegion));

      manager.clear(testCacheRegion);
      manager.clear(memoryCacheRegion);

      WebPortalLogger.info("****   TEST METHOD: testMultipleCacheRegions() completed.      ****");
   }

   public void testCacheExpiry() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testCacheExpiry() started.        ****");

      List list = null;
      String key = "TEST123";
      list = (LinkedList)manager.getCacheObject(key, this);

      try {
         timeToSleep = 7000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      list = (LinkedList)manager.getCacheObject(key, this);

      try {
         timeToSleep = 2000;
         WebPortalLogger.info("Sleep for " + timeToSleep + " ms");
         Thread.sleep(timeToSleep);
      } catch(Exception e) {
         WebPortalLogger.info("Exception in Thread.sleep()..");
      }

      list = (LinkedList)manager.getCacheObject(key, this);

      assertTrue(true);
      manager.clear();

      WebPortalLogger.info("****   TEST METHOD: testCacheExpiry() completed.      ****");
   }

   public void testCachingUsingCustomCacheKey() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testCachingUsingTestCacheKey() started.        ****");

      TestCacheKey cacheKey = new TestCacheKey();
      String key = cacheKey.getCacheKey().toString();

      WebPortalLogger.info("Got key from CacheKey class. Key : " + key);

      try {
         LinkedList list = buildList();
         manager.putCacheObject(key, list);
         assertEquals(list, manager.getCacheObject(key, this));
         manager.clear();
      } catch (Exception e) {
         e.printStackTrace();
         throw new CachingException(e);
      }
      WebPortalLogger.info("****   TEST METHOD: testCachingUsingTestCacheKey() completed.      ****");
   }

   public void testStoreRetrieveClear(int iterations) throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testStoreRetrieveClear10K() started.        ****");

      long start = System.currentTimeMillis();

      Serializable key = "CACHE KEY";
      Serializable value = "CACHE OBJECT";

      System.out.println("Iterations: " + iterations);

      for (int i=0; i<iterations; i++) {
         key   = key   + Integer.toString(i);
         value = value + Integer.toString(i);
         WebPortalLogger.info("Before putCacheObject : " +  i + ":" + manager.isPresent(key));
         assertTrue(!manager.isPresent(key));
         // Now store the object in cache
         manager.putCacheObject(key, value);
         WebPortalLogger.info("After putCacheObject : " +  i + ":" + manager.isPresent(key));
         assertTrue(manager.isPresent(key));
         assertEquals(value, manager.peek(key));
         // And now remove the object from cache
         manager.removeCacheObject(key);
         WebPortalLogger.info("After removeCacheObject : " +  i + ":" + manager.isPresent(key));
         assertTrue(!manager.isPresent(key));
      }

      long end = System.currentTimeMillis();
      WebPortalLogger.info("Total Time Taken=" + (end-start) + "ms");
      System.out.println("Total Time Taken=" + (end-start) + "ms");

      WebPortalLogger.info("****   TEST METHOD: testStoreRetrieveClear10K() completed.      ****");
   }

   public void testMultipleStoreRetrieveClear() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testMultipleStoreRetrieveClear() started.        ****");

      long start = System.currentTimeMillis();

      int it = 0;
      Serializable key = "CACHE KEY";
      Serializable value = "CACHE OBJECT";

      // Iterate 2000 times
      it = 2000;
      testStoreRetrieveClear(it);

      // Iterate 4000 times
      it = 4000;
      testStoreRetrieveClear(it);

      // Iterate 6000 times
      it = 6000;
      testStoreRetrieveClear(it);

      WebPortalLogger.info("****   TEST METHOD: testMultipleStoreRetrieveClear() completed.      ****");
   }

   /***
    * Adds items to cache, gets them, and removes them. The item count is more
    * than the size of the memory cache, so items should be dumped.
    *
    * @exception CachingException If an error occurs
    */
   public void testLRUMemoryCache() throws CachingException {
      WebPortalLogger.info("****   TEST METHOD: testLRUMemoryCache() started.        ****");
      int items = 200;

      try {
         // Add items to cache
         for (int i = 0; i < items; i++) {
            manager.putCacheObject(i + ":key", memoryCacheRegion + " data " + i, memoryCacheRegion);
         }

//         manager.listCacheKeys(memoryCacheRegion);

         // Test that initial items have been purged
         for (int i = 0; i < 102; i++) {
            if (manager.peek(i + ":key", memoryCacheRegion) == null) {
               WebPortalLogger.info(i + "  -  NULL");
            } else {
               WebPortalLogger.info(i + "  -  NOT NULL");
            }
            assertNull(manager.peek(i + ":key", memoryCacheRegion));
         }

         // Test that last items are in cache
         for (int i = 102; i < items; i++) {
            if (manager.peek(i + ":key", memoryCacheRegion) == null) {
               WebPortalLogger.info(i + "  -  NULL");
            } else {
               WebPortalLogger.info(i + "  -  NOT NULL");
            }
            assertNotNull(manager.peek(i + ":key", memoryCacheRegion));
         }

         // Remove all the items
         for (int i = 0; i < items; i++) {
            manager.removeCacheObject(i + ":key", memoryCacheRegion);
         }

         // Verify removal
         WebPortalLogger.info("After removing all the elements from cache:");
         for (int i = 0; i < items; i++) {
            if (manager.peek(i + ":key", memoryCacheRegion) == null) {
               WebPortalLogger.info(i + "  -  NULL");
            } else {
               WebPortalLogger.info(i + "  -  NOT NULL");
            }
            assertNull("Removed key should be null: " + i + ":key", manager.peek(i + ":key", memoryCacheRegion));
         }
      } catch (Exception e) {
         e.printStackTrace();
         throw new CachingException(e);
      }

      WebPortalLogger.info("****   TEST METHOD: testLRUMemoryCache() completed.      ****");
   }

}
// End of CachingTestCase.java
